<?php

namespace ThemeHouse\UserNotes\Import\Importer;

use Exception;
use XF\Db\Mysqli\Adapter;
use XF\Import\StepState;
use XF\Timer;

class Z61MemberNotes extends \XF\Import\Importer\AbstractForumImporter
{

	/**
	 * @var mixed|string|Adapter
	 */
	protected $sourceDb;

	public static function getListInfo()
	{
		return [
			'target' => '[OzzModz] User Profile Notes',
			'source' => 'Z61 - Member Notes',
			'beta' => true
		];
	}

	protected function getBaseConfigDefault()
	{
		return [
			'db' => [
				'host' => '127.0.0.1',
				'username' => '',
				'password' => '',
				'dbname' => '',
				'port' => 3306,
			],
			'notes_per_stage' => 50,
		];
	}

	public function renderStepConfigOptions(array $vars)
	{
		return '';
	}

	public function renderBaseConfigOptions(array $vars)
	{
		return $this->app->templater()->renderTemplate('admin:th_usernotes_import_z61_membernotes_config', $this->getBaseConfigDefault());
	}

	public function validateBaseConfig(array &$baseConfig, array &$errors)
	{
		$fullConfig = array_replace_recursive($this->getBaseConfigDefault(), $baseConfig);
		$validDbConnection = false;

		if ($fullConfig['db']['host'])
		{
			try
			{
				$sourceDb = new Adapter($fullConfig['db'], false);
				$sourceDb->getConnection();
				$validDbConnection = true;
			}
			catch (Exception $e)
			{
				$errors[] = \XF::phrase('source_database_connection_details_not_correct_x', ['message' => $e->getMessage()]);
				return !$errors;
			}

			$quotesTable = $sourceDb->rawQuery("SHOW TABLES FROM `{$fullConfig['db']['dbname']}` LIKE 'xf_z61_member_note'");

			if (!$quotesTable)
			{
				$errors[] = \XF::phrase('source_database_connection_details_not_correct_x', ['message' => $fullConfig['db']['dbname']]);

				if ($fullConfig['db']['dbname'] === '')
				{
					$errors[] = \XF::phrase('please_enter_database_name');
				}
			}
		}

		if (!$validDbConnection || !$fullConfig['db']['host'] || !$fullConfig['notes_per_stage'])
		{
			$errors[] = \XF::phrase('please_complete_required_fields');
		}

		return !$errors;
	}

	protected function getStepConfigDefault()
	{
		return [
			'notes' => []
		];
	}

	public function validateStepConfig(array $steps, array &$stepConfig, array &$errors)
	{
		return true;
	}

	public function getSteps()
	{
		return [
			'notes' => [
				'title' => \XF::phrase('th_usernotes_notes')
			],
		];
	}

	protected function doInitializeSource()
	{
		$dbConfig = $this->baseConfig['db'];

		$this->sourceDb = new Adapter($dbConfig, false);
	}

	// ################################################ NOTES ################################################

	public function getStepEndNotes()
	{
		return (int) $this->sourceDb->fetchOne("SELECT MAX(note_id) AS notes FROM xf_z61_member_note") ?: 0;
	}

	public function stepNotes(StepState $state, array $stepConfig, $maxTime)
	{
		$timer = new Timer($maxTime);
		$onStage = (int) $this->baseConfig['notes_per_stage'];

		$oldNotes = $this->sourceDb->fetchAll("
			SELECT *
			FROM xf_z61_member_note
			WHERE note_id > ? AND note_id <= ?
			LIMIT $onStage
		", [$state->startAfter, $state->end]);

		if (!$oldNotes)
		{
			return $state->complete();
		}

		foreach ($oldNotes as $oldNote)
		{
			$state->startAfter = $oldNote['note_id'];

			/** @var \ThemeHouse\UserNotes\Entity\UserNote $note */
			$note = $this->em()->create('ThemeHouse\UserNotes:UserNote');
			$this->setupImportQuote($note, $oldNote, $stepConfig);
			if (!$note->save())
			{
				continue;
			}

			$state->imported++;

			if ($timer->limitExceeded())
			{
				break;
			}
		}

		return $state->resumeIfNeeded();
	}

	protected function setupImportQuote(\ThemeHouse\UserNotes\Entity\UserNote $note, array $oldQuote, array $stepConfig)
	{
		$note->profile_user_id = $oldQuote['note_user_id'];
		$note->user_id = $oldQuote['user_id'];
		$note->username = $oldQuote['username'];
		$note->post_date = $oldQuote['post_date'];
		$note->last_edit_date = $oldQuote['last_edit_date'];

		$note->set('message', html_entity_decode(strip_tags($oldQuote['message'])));
	}

	public function getFinalizeJobs(array $stepsRun)
	{
		return [];
	}

}