<?php

namespace ThemeHouse\UserNotes\Pub\Controller;

use ThemeHouse\UserNotes\Entity\UserNote;
use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

/**
 * Class UserNotes
 * @package ThemeHouse\UserNotes\Pub\Controller
 */
class UserNotes extends AbstractController
{
	/**
	 * @param $action
	 * @param ParameterBag $params
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function preDispatchController($action, ParameterBag $params)
	{
		if (!\XF::visitor()->hasPermission('th_usernotes', 'view'))
		{
			throw $this->exception($this->noPermission());
		}
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionIndex(ParameterBag $params)
	{
		$user = null;
		if ($params->user_id)
		{
			$user = $this->assertUserExists($params->user_id);
		}

		$page = $this->filterPage();
		$perPage = 25;

		$noteFinder = $this->finder('ThemeHouse\UserNotes:UserNote')
			->order('post_date', 'DESC');

		if ($user)
		{
			$noteFinder->where('profile_user_id', '=', $user->user_id);
		}

		$this->applyFilters($noteFinder);

		$noteFinder->limitByPage($page, $perPage);
		$filters = $this->getFilterInput(true);

		$notes = $noteFinder->fetch();

		$viewParams = [
			'user' => $user,
			'notes' => $notes->filterViewable(),
			'filters' => $filters,

			'page' => $page,
			'perPage' => $perPage,
			'total' => $noteFinder->total()
		];

		return $this->view('ThemeHouse\UserNotes:UserNotes', 'th_usernotes_list', $viewParams);
	}

	public function actionRefineSearch(ParameterBag $params)
	{
		if (!\XF::visitor()->hasPermission('th_usernotes', 'search'))
		{
			return $this->noPermission();
		}

		$input = $this->getFilterInput();

		if ($params->user_id)
		{
			$user = $this->assertUserExists($params->user_id);
		}
		else
		{
			/** @var \XF\Entity\User $user */
			$user = $this->em()->findOne('XF:User', ['username' => $input['profile_username']]);
		}

		$viewParams = [
			'user' => $user,
			'datePresets' => \XF::language()->getDatePresets(),
			'conditions' => $this->getFilterInput(),
		];

		return $this->view(
			'ThemeHouse\UserNotes:UserNotes\RefineSearch',
			'th_usernotes_refine_search',
			$viewParams
		);
	}

	protected function getFilterInput($removeEmpty = false): array
	{
		$input = $this->filter([
			'profile_username' => 'str',
			'username' => 'str',
			'message' => 'str',
			'start' => 'datetime',
			'end' => 'datetime',
		]);

		if ($removeEmpty)
		{
			$input = array_filter($input);
		}

		return $input;
	}

	protected function applyFilters(\XF\Mvc\Entity\Finder $finder)
	{
		$filters = $this->getFilterInput();

		if (!\XF::visitor()->hasPermission('th_usernotes', 'search'))
		{
			return;
		}

		if ($filters['profile_username'])
		{
			$finder->where('ProfileUser.username', 'LIKE', $finder->escapeLike($filters['profile_username'], '%?%'));
		}
		if ($filters['username'])
		{
			$finder->where('User.username', 'LIKE', $finder->escapeLike($filters['username'], '%?%'));
		}
		if ($filters['message'])
		{
			$finder->where('message', 'LIKE', $finder->escapeLike($filters['message'], '%?%'));
		}

		if ($filters['start'])
		{
			$finder->where('post_date', '>', $filters['start']);
		}
		if ($filters['end'])
		{
			$finder->where('post_date', '<', $filters['end']);
		}
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\AbstractReply|\XF\Mvc\Reply\Redirect|\XF\Mvc\Reply\View
	 * @throws \XF\PrintableException
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionSave(ParameterBag $params)
	{
		$user = $this->assertUserExists($params->user_id);

		if ($params->user_note_id)
		{
			$userNote = $this->assertUserNoteExists($params->user_note_id);
			if (!$userNote->canEdit())
			{
				return $this->noPermission();
			}
		}
		else
		{
			if (!\XF::visitor()->hasPermission('th_usernotes', 'add'))
			{
				return $this->noPermission();
			}

			/** @var UserNote $userNote */
			$userNote = $this->em()->create('ThemeHouse\UserNotes:UserNote');
			$userNote->profile_user_id = $params->user_id;
		}

		$this->userNoteSaveProcess($userNote)->run();


		if ($this->filter('_xfWithData', 'bool') && $this->filter('_xfInlineEdit', 'bool'))
		{
			$viewParams = [
				'note' => $userNote,
			];
			$reply = $this->view('ThemeHouse\UserNotes:UserNote\Edit', 'th_usernotes_edit_new_note', $viewParams);
			$reply->setJsonParam('message', \XF::phrase('your_changes_have_been_saved'));
			return $reply;
		}
		else
		{
			return $this->redirect($this->buildLink('members', $user) . '#th-usernotes');
		}
	}

	/**
	 * @param UserNote $note
	 * @return \XF\Mvc\FormAction
	 */
	protected function userNoteSaveProcess(UserNote $note)
	{
		$form = $this->formAction();

		$input = [];

		if ($note->isInsert())
		{
			$input += [
				'username' => \XF::visitor()->username,
				'user_id' => \XF::visitor()->user_id
			];
		}
		else
		{
			$input += [
				'last_edit_date' => \XF::$time
			];
		}

		/** @var \XF\ControllerPlugin\Editor $editor */
		$editor = $this->plugin('XF:Editor');
		$input['message'] = $editor->fromInput('message');

		$form->basicEntitySave($note, $input);

		return $form;
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\AbstractReply|\XF\Mvc\Reply\View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionEdit(ParameterBag $params)
	{
		$userNote = $this->assertUserNoteExists($params->user_note_id);

		if (!$userNote->canEdit())
		{
			return $this->noPermission();
		}


		$viewParams = [
			'note' => $userNote,
			'profileUser' => $userNote->ProfileUser,

			'quickEdit' => $this->filter('_xfWithData', 'bool')
		];
		return $this->view('ThemeHouse\UserNotes:UserNote\Edit', 'th_usernotes_edit', $viewParams);
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\Error|\XF\Mvc\Reply\Redirect|\XF\Mvc\Reply\View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionDelete(ParameterBag $params)
	{
		$userNote = $this->assertUserNoteExists($params->user_note_id);

		/** @var \XF\ControllerPlugin\Delete $plugin */
		$plugin = $this->plugin('XF:Delete');

		return $plugin->actionDelete(
			$userNote,
			$this->buildLink('members/th-usernotes/delete', $userNote),
			$this->buildLink('members', $userNote->ProfileUser) . '#th-usernotes',
			$this->buildLink('members', $userNote->ProfileUser) . '#th-usernotes',
			\XF::phrase('th_usernotes_note')
		);
	}

	/**
	 * @param $id
	 * @param null $with
	 * @return \XF\Entity\User
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertUserExists($id, $with = null): \XF\Entity\User
	{
		/** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->assertRecordExists('XF:User', $id, $with, 'requested_user_not_found');
	}

	/**
	 * @param $id
	 * @param null $with
	 * @param null $phraseKey
	 * @return UserNote
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertUserNoteExists($id, $with = null, $phraseKey = null): UserNote
	{
		/** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->assertRecordExists('ThemeHouse\UserNotes:UserNote', $id, $with, $phraseKey);
	}
}